import {
    SELECT_CUSTOMER_INVOICES,
    SELECT_INVOICE,
    SELECT_MOVE_INVOICE,
    ADD_MOVE_IN_INVOICE,
    FILTER_PAYMENTS_INVOICE,
    RESET_INVOICES
} from '@/constants/actionTypes'

import {
    SET_INVOICES,
    SET_SELECTED_INVOICE,
    SET_SELECTED_CURRENCY_INVOICE,
    SET_MOVE_IN_INVOICE,
    SET_FILTER_PAYMENTS_INVOICE
} from '@/constants/mutationTypes'

const initialState = {
    invoices: [],
    filterPaymentsInvoice: [],
    currencyInvoice: [],
    selectedInvoice : null
}

const state = {
    invoices: initialState.invoices,
    currencyInvoice: initialState.currencyInvoice,
    selectedInvoice: initialState.selectedInvoice,
    filterPaymentsInvoice: initialState.filterPaymentsInvoice
}

const getters = {
    /**
     * [invoices description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    invoices(state) {
        return state.invoices
    },
    /**
     * [currencyInvoice description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    currencyInvoice(state) {
        return state.currencyInvoice
    },
    /**
     * [selectedInvoice description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    selectedInvoice(state) {
        return state.selectedInvoice
    },
    /**
     * [invoicesVisible]
     * @method invoicesVisible
     * @param  {[type]}        state [description]
     * @return {[type]}              [description]
     */
    invoicesVisible(state) {
        return state.filterPaymentsInvoice.length === 0 ?state.invoices : state.filterPaymentsInvoice
    }
}

const mutations = {
    /**
     * [invoices description]
     * @type {[type]}
     * @param  {[type]} state [description]
     * @param  {[type]} payload [description]
     */
    [SET_INVOICES] (state, payload) {
        state.invoices = payload
    },
    /**
     * [selectedInvoice description]
     * @type {[type]}
     * @param  {[type]} state [description]
     * @param  {[type]} payload [description]
     */
    [SET_SELECTED_INVOICE] (state, payload) {
        state.selectedInvoice = payload
    },
    /**
     * [currencyInvoice description]
     * @type {[type]}
     * @param  {[type]} state [description]
     * @param  {[type]} payload [description]
     */
    [SET_SELECTED_CURRENCY_INVOICE] (state, payload) {
        state.currencyInvoice = payload
    },
    /**
     * [moveLines description]
     * @type {Array}
     * @param  {[type]} state [description]
     * @param  {[type]} payload [description]
     */
    [SET_MOVE_IN_INVOICE] (state, payload) {
        if (payload.mode === 'full') {
            payload.moveLine.forEach(moves => {
                let movesFound = state.selectedInvoice.moveLines.find(item => item.id === moves.id)

                if (movesFound) return

                state.selectedInvoice.moveLines = [moves, ...state.selectedInvoice.moveLines]
            })
        } else {

            let movesFound = state.selectedInvoice.moveLines.find(item => item.id === payload.moveLine.id)
            if (movesFound) return
            state.selectedInvoice.moveLines = [payload.moveLine, ...state.selectedInvoice.moveLines]
        }
    },
    /**
     * [filterPaymentsInvoice description]
     * @type {[type]}
     * @param  {[type]} state [description]
     * @param  {[type]} payload [description]
     */
    [SET_FILTER_PAYMENTS_INVOICE] (state, payload) {
        return state.filterPaymentsInvoice = payload
    }
}

const actions = {
    /**
     * [Selecciona las factura del cliente seleccionado]
     * @param {*} param0
     * @param {*} payload
     */
    [SELECT_CUSTOMER_INVOICES] ({ commit }, payload) {
        commit(SET_INVOICES, payload)
    },
    /**
     * [Seleccionar una factura]
     * @param {*} param0
     * @param {*} payload
     */
    [SELECT_INVOICE] ({ commit, dispatch }, payload) {
        commit(SET_SELECTED_INVOICE, payload)
        commit(SET_SELECTED_CURRENCY_INVOICE, payload.currency)
        dispatch(SELECT_MOVE_INVOICE, payload.moveLines)
    },
    /**
     *
     */
    [ADD_MOVE_IN_INVOICE] ({ commit }, payload) {
        commit(SET_MOVE_IN_INVOICE, payload)
    },
    /**
     *
     * @param {*} param0
     * @param {*} payload
     */
    [FILTER_PAYMENTS_INVOICE] ({ commit }, payload ){
        commit(SET_FILTER_PAYMENTS_INVOICE, payload)
    },
    [RESET_INVOICES] ({ commit }) {
        commit(SET_INVOICES, [] )
        commit(SET_SELECTED_INVOICE, null)
        commit(SET_SELECTED_CURRENCY_INVOICE, [] )
        commit(SET_FILTER_PAYMENTS_INVOICE, [] )
    }
 }

export default {
    state,
    getters,
    mutations,
    actions
}
