/* ACTIONS */
import {
    CHANGE_INITIAL_PAID,
    RESET_PAYMENTS,
    CALCULATE_CURRENCY_TOTAL,
    SELECT_PAYMENTS_BANK,
    SELECT_PAYMENTS_BANK_TYPE,
    CHANGE_PAYMENTS_BANK_REF,
    CHANGE_PAYMENTS_BANK_DATE_MATURITY,
    CHANGE_PAYMENTS_BANK_NUMBER_CTA,
    CHANGE_PAYMENTS_BANK_NAME_HOLDER,
    RESET_SELECTED_JOURNAL,
    RESET_PAYMENTS_BANK
} from '@/constants/actionTypes'
/* MUTATIONS */
import {
    SET_PAID_TOTAL,
    SET_CALCULATE_CURRENCY_TOTAL,
    SET_CURRENCY_TOTAL,
    SET_SELECTED_BANK,
    SET_SELECTED_BANK_TYPE,
    SET_PAYMENTS_BANK_REF,
    SET_PAYMENTS_BANK_DATE_MATURITY,
    SET_PAYMENTS_BANK_NUMBER_CTA,
    SET_PAYMENTS_BANK_NAME_HOLDER
} from '@/constants/mutationTypes'

const initialState = {
    paidTotal: 0,
    paymentsReturned: 0,
    paymentCurrencyTotal: 0,
    selectedPaymentsBank: null,
    selectedPaymentsBankType: null,
    paymentsBankRef: '',
    paymentsBankDateMaturity: '',
    paymentsBankNumberCta: '',
    paymentsBankNameHolder: ''
}
const state = {
    paidTotal: initialState.paidTotal,
    paymentsReturned: initialState.paymentsReturned,
    paymentCurrencyTotal: initialState.paymentCurrencyTotal,
    selectedPaymentsBank: initialState.selectedPaymentsBank,
    selectedPaymentsBankType: initialState.selectedPaymentsBankType,
    paymentsBankRef: initialState.paymentsBankRef,
    paymentsBankDateMaturity: initialState.paymentsBankDateMaturity,
    paymentsBankNumberCta: initialState.paymentsBankNumberCta,
    paymentsBankNameHolder: initialState.paymentsBankNameHolder

}

const getters = {
    /**
     * [paidTotal description]
     * @method paidTotal
     * @param  {[type]}     state [description]
     * @return {[type]}     [description]
     */
    paidTotal (state) {
        return state.paidTotal
    },
    /**
     * [paymentsReturned description]
     * @method paymentsReturned
     * @param  {[type]}         state [description]
     * @return {[type]}               [description]
     */
    paymentsReturned (state) {
        return state.paymentsReturned
    },
    /**
     * [paymentCurrencyTotal description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentCurrencyTotal (state) {
        return state.paymentCurrencyTotal
    },
    /**
     * [selectedPaymentsBank description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    selectedPaymentsBank(state) {
        return state.selectedPaymentsBank
    },
    /**
     * [selectedPaymentsBankType description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    selectedPaymentsBankType(state) {
        return state.selectedPaymentsBankType
    },
    /**
     * [paymentsBankRef description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsBankRef (state) {
        return state.paymentsBankRef
    },
    /**
     * [paymentsBankDateMaturity description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsBankDateMaturity (state) {
        return state.paymentsBankDateMaturity
    },
    /**
     * [paymentsBankNumberCta description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsBankNumberCta ( state ) {
        return state.paymentsBankNumberCta
    },
    /**
     * [paymentsBankNameHolder description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsBankNameHolder( state ) {
        return state.paymentsBankNameHolder
    }
}

const mutations = {
    /**
     * [paidTotal description]
     * @type {[type]}
     */
    [SET_PAID_TOTAL] (state, payload) {
        state.paidTotal = payload
    },
    /**
     * [paymentCurrencyTotal description]
     * @type {[type]}
     */
    [SET_CURRENCY_TOTAL] (state, payload) {
        state.paymentCurrencyTotal = payload
    },
    /**
     * [SET_CALCULATE_CURRENCY_TOTAL]
     * @type {[type]}
     */
    [SET_CALCULATE_CURRENCY_TOTAL] (state, payload) {
        let currencyJournal = payload.movesCurrencyAmount.find(item => item.id ===payload.currency.id)
        let total = currencyJournal.amount
        if (!currencyJournal)
            total = 0
        state.paymentCurrencyTotal = total
    },
    /**
     * [selectedPaymentsBank description]
     * @type {[type]}
     */
    [SET_SELECTED_BANK] (state, payload) {
        state.selectedPaymentsBank = payload
    },
    /**
     * [selectedPaymentsBankType description]
     * @type {[type]}
     */
    [SET_SELECTED_BANK_TYPE] (state, payload) {
        state.selectedPaymentsBankType = payload
    },
    /**
     * [paymentsBankRef description]
     * @type {[type]}
     */
    [SET_PAYMENTS_BANK_REF] (state, payload) {
        state.paymentsBankRef = payload
    },
    /**
     * [paymentsBankDateMaturity description]
     * @type {[type]}
     */
    [SET_PAYMENTS_BANK_DATE_MATURITY] (state, payload) {
        state.paymentsBankDateMaturity = payload
    },
    /**
     * [paymentsBankNumberCta description]
     * @type {[type]}
     */
    [SET_PAYMENTS_BANK_NUMBER_CTA] (state, payload) {
        state.paymentsBankNumberCta =  payload
    },
    /**
     * [actions description]
     * @type {Object}
     */
    [SET_PAYMENTS_BANK_NAME_HOLDER] (state, payload) {
        state.paymentsBankNameHolder = payload
    }
}

const actions = {
    /**
     * [CHANGE_INITIAL_PAID]
     */
    [CHANGE_INITIAL_PAID] ({ commit },payload) {
        commit(SET_PAID_TOTAL, payload)
    },
    /**
     * [CALCULATE_CURRENCY_TOTAL]
     */
    [CALCULATE_CURRENCY_TOTAL] ({commit, getters}, payload) {
        let currency = []
        if (payload.currency.id){
            currency = payload.currency
        } else {
            currency = getters.currencyCompany
        }
        commit(SET_CALCULATE_CURRENCY_TOTAL, {
            currency: currency,
            movesCurrencyAmount: getters.currencyAmount
        })
        commit(SET_PAID_TOTAL, 0)
    },
    /**
     * [SELECT_PAYMENTS_BANK]
     */
    [SELECT_PAYMENTS_BANK] ({ commit }, payload){
        commit(SET_SELECTED_BANK, payload)
    },
    /**
     * [SELECT_PAYMENTS_BANK_TYPE]
     */
    [SELECT_PAYMENTS_BANK_TYPE] ({ commit }, payload) {
        commit(SET_SELECTED_BANK_TYPE, payload)
    },
    /**
     * [CHANGE_PAYMENTS_BANK_REF]
     */
    [CHANGE_PAYMENTS_BANK_REF] ({ commit }, payload) {
            commit(SET_PAYMENTS_BANK_REF, payload)
    },
    /**
     * [CHANGE_PAYMENTS_BANK_DATE_MATURITY]
     */
    [CHANGE_PAYMENTS_BANK_DATE_MATURITY] ({ commit }, payload) {
        commit(SET_PAYMENTS_BANK_DATE_MATURITY, payload)
    },
    /**
     * [CHANGE_PAYMENTS_BANK_NUMBER_CTA]
     */
    [CHANGE_PAYMENTS_BANK_NUMBER_CTA] ({ commit }, payload) {
        commit(SET_PAYMENTS_BANK_NUMBER_CTA, payload)
    },
    /**
     * [CHANGE_PAYMENTS_BANK_NAME_HOLDER]
     */
    [CHANGE_PAYMENTS_BANK_NAME_HOLDER] ({ commit }, payload) {
        commit(SET_PAYMENTS_BANK_NAME_HOLDER, payload)
    },
    /**
     * [RESET_PAYMENTS_BANK]
     */
    [RESET_PAYMENTS_BANK] ({commit, dispatch}) {
        commit(SET_SELECTED_BANK, null)
        commit(SET_SELECTED_BANK_TYPE, null)
        commit(SET_PAYMENTS_BANK_REF, '')
        commit(SET_PAYMENTS_BANK_DATE_MATURITY, '')
        commit(SET_PAYMENTS_BANK_NUMBER_CTA, '')
        commit(SET_PAYMENTS_BANK_NAME_HOLDER, '')

        commit(SET_PAID_TOTAL, 0)
        commit(SET_CURRENCY_TOTAL, 0)
        
        dispatch(RESET_SELECTED_JOURNAL)
    },
    /**
     * [RESET_PAYMENTS]
     */
    [RESET_PAYMENTS] ({ commit }) {
        commit(SET_PAID_TOTAL, 0)
        commit(SET_CURRENCY_TOTAL, 0)
        commit(SET_SELECTED_BANK, null)
        commit(SET_SELECTED_BANK_TYPE, null)
        commit(SET_PAYMENTS_BANK_REF, '')
        commit(SET_PAYMENTS_BANK_DATE_MATURITY, '')
        commit(SET_PAYMENTS_BANK_NUMBER_CTA, '')
        commit(SET_PAYMENTS_BANK_NAME_HOLDER, '')
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}
